package dev.micle.loginprotection.network;

import dev.micle.loginprotection.LoginProtection;
import dev.micle.loginprotection.network.client.InputPacket;
import dev.micle.loginprotection.network.client.LastInputTickPacket;
import dev.micle.loginprotection.network.server.PlayerStatePacket;
import dev.micle.loginprotection.network.server.RequestLastInputTickPacket;
import net.minecraftforge.network.NetworkRegistry;
import net.minecraftforge.network.simple.SimpleChannel;

public class NetworkManager {
    private static SimpleChannel channel;

    public static void init() {
        // Create channel
        channel = NetworkRegistry.ChannelBuilder.named(LoginProtection.createResourceLocation("network"))
                .clientAcceptedVersions(v -> v.equals(LoginProtection.getVersion()))
                .serverAcceptedVersions(v -> v.equals(LoginProtection.getVersion()))
                .networkProtocolVersion(LoginProtection::getVersion)
                .simpleChannel();

        // Register packets
        int id = 0;
        channel.messageBuilder(InputPacket.class, id++)
                .encoder(InputPacket::encode)
                .decoder(InputPacket::decode)
                .consumerNetworkThread(InputPacket::handle)
                .add();
        channel.messageBuilder(PlayerStatePacket.class, id++)
                .encoder(PlayerStatePacket::encode)
                .decoder(PlayerStatePacket::decode)
                .consumerNetworkThread(PlayerStatePacket::handle)
                .add();
        channel.messageBuilder(RequestLastInputTickPacket.class, id++)
                .encoder(RequestLastInputTickPacket::encode)
                .decoder(RequestLastInputTickPacket::decode)
                .consumerNetworkThread(RequestLastInputTickPacket::handle)
                .add();
        channel.messageBuilder(LastInputTickPacket.class, id++)
                .encoder(LastInputTickPacket::encode)
                .decoder(LastInputTickPacket::decode)
                .consumerNetworkThread(LastInputTickPacket::handle)
                .add();
    }

    public static SimpleChannel getChannel() {
        return channel;
    }
}
